package javax.net.ssl;

import java.util.List;
import java.util.function.BiFunction;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.io.IOException;
import java.net.Socket;

public abstract class SSLSocket extends Socket
{
    protected SSLSocket() {
    }
    
    protected SSLSocket(String host, int port) throws IOException, UnknownHostException {
        super(host, port);
    }
    
    protected SSLSocket(InetAddress address, int port) throws IOException {
        super(address, port);
    }
    
    protected SSLSocket(String host, int port, InetAddress localAddr, int localPort) throws IOException, UnknownHostException {
        super(host, port, localAddr, localPort);
    }
    
    protected SSLSocket(InetAddress address, int port, InetAddress localAddr, int localPort) throws IOException {
        super(address, port, localAddr, localPort);
    }
    
    public abstract String[] getSupportedCipherSuites();
    
    public abstract String[] getEnabledCipherSuites();
    
    public abstract void setEnabledCipherSuites(String[] p0);
    
    public abstract String[] getSupportedProtocols();
    
    public abstract String[] getEnabledProtocols();
    
    public abstract void setEnabledProtocols(String[] p0);
    
    public abstract SSLSession getSession();
    
    public SSLSession getHandshakeSession() {
        throw new UnsupportedOperationException();
    }
    
    public abstract void addHandshakeCompletedListener(HandshakeCompletedListener p0);
    
    public abstract void removeHandshakeCompletedListener(HandshakeCompletedListener p0);
    
    public abstract void startHandshake() throws IOException;
    
    public abstract void setUseClientMode(boolean p0);
    
    public abstract boolean getUseClientMode();
    
    public abstract void setNeedClientAuth(boolean p0);
    
    public abstract boolean getNeedClientAuth();
    
    public abstract void setWantClientAuth(boolean p0);
    
    public abstract boolean getWantClientAuth();
    
    public abstract void setEnableSessionCreation(boolean p0);
    
    public abstract boolean getEnableSessionCreation();
    
    public SSLParameters getSSLParameters() {
        SSLParameters sslParameters = new SSLParameters();
        sslParameters.setCipherSuites(this.getEnabledCipherSuites());
        sslParameters.setProtocols(this.getEnabledProtocols());
        if (this.getNeedClientAuth()) {
            sslParameters.setNeedClientAuth(true);
        }
        else if (this.getWantClientAuth()) {
            sslParameters.setWantClientAuth(true);
        }
        return sslParameters;
    }
    
    public void setSSLParameters(SSLParameters sslParameters) {
        String[] cipherSuites = sslParameters.getCipherSuites();
        if (cipherSuites != null) {
            this.setEnabledCipherSuites(cipherSuites);
        }
        String[] protocols = sslParameters.getProtocols();
        if (protocols != null) {
            this.setEnabledProtocols(protocols);
        }
        if (sslParameters.getNeedClientAuth()) {
            this.setNeedClientAuth(true);
        }
        else if (sslParameters.getWantClientAuth()) {
            this.setWantClientAuth(true);
        }
        else {
            this.setWantClientAuth(false);
        }
    }
    
    public String getApplicationProtocol() {
        throw new UnsupportedOperationException();
    }
    
    public String getHandshakeApplicationProtocol() {
        throw new UnsupportedOperationException();
    }
    
    public void setHandshakeApplicationProtocolSelector(BiFunction<SSLSocket, List<String>, String> biFunction) {
        throw new UnsupportedOperationException();
    }
    
    public BiFunction<SSLSocket, List<String>, String> getHandshakeApplicationProtocolSelector() {
        throw new UnsupportedOperationException();
    }
}
